import { Component, OnInit, Input, Output, EventEmitter } from '@angular/core';
import { NzMessageService, UploadFile } from 'ng-zorro-antd';

@Component({
    selector: 'sn-img-upload',
    templateUrl: './img-upload.component.html',
    styleUrls: ['./img-upload.component.css']
})
export class ImgUploadComponent implements OnInit {

    snAction: string;
    previewImage: string = '';
    previewVisible: boolean = false;
    uploadData: any = {"bucketAlias": "SmartEvent"};
    _snFileList: any[] = [];
    imgList: any[] = [];
    @Input() set uploadUrl(url) {
        this.snAction = url + "/foundation/aliyun/api/oss/upload";
    }
    @Input() set snFileList(list){
        // console.log('@Input() snFileList==', list);
        if(!list) return;
        if(this.imgList.length > 0){
            let flag = true;
            this.imgList.forEach((item) => {
                // console.log(list.indexOf(item));
                if(list.indexOf(item) == -1){
                    flag = false;
                }
            });
            if(flag) return;
        }
        this.imgList = list;
        let imglist = [];
        list.forEach((item) => {
            imglist.push({
                name: item.name,
                fileKey: item.fileKey,
                status: item.status ? item.status : "done",
                url: item.uri,
                thumbnUrl: item.thumbnail
            });
            
        });
        this._snFileList = [...imglist];
    };
    @Input() snText: string;
    // @Input() _snFileList: IMGLIST[];
    @Output() snFileListChange = new EventEmitter();
    @Output() snChangeCallback = new EventEmitter();
    @Output() snRemoveCallback = new EventEmitter();
    @Output() snUploadCallback = new EventEmitter();

    constructor(private msg: NzMessageService) {
        this.snText = "上传图片";
    }

    ngOnInit() {

    }

    handlePreview = (file: UploadFile) => {
        // console.log('handlePreview==', file);
        // this.previewImage = file.url || file.thumbUrl;
        // this.previewVisible = true;

        // console.log('handlePreview=', file);
        
        // let previewImage = file.response.content.url || file.response.content.urlThumbnail;
        let previewImage = file.url || file.thumbUrl;

        window.open(previewImage, "_blank");
    }

    beforeUpload = (file: File) => {
        // console.log('beforeUpload==', file);
        // console.log('beforeUpload=snFileList=', this.snFileList);
        const isIMG = file.type === 'image/jpeg' || file.type === 'image/png';
        if (!isIMG) {
            this.msg.error('只支持JPG或PNG格式!');
        }
        const isLt2M = file.size / 1024 / 1024 < 10;
        if (!isLt2M) {
            this.msg.error('上传文件需小于10MB!');
        }
        return isIMG && isLt2M;
    }

    changeCallback(event) {
        if(event.type === "success") {
            // console.log('changeCallback=event=', event);
            this.imgList = [...this.imgList, {
                fileKey: event.file.response.content.fileKey,
                thumbnail: event.file.response.content.urlThumbnail,
                name: event.file.name,
                status: event.file.status,
                uri: event.file.response.content.url,
            }];
            this.snChangeCallback.emit(this.imgList);
            this.snUploadCallback.emit(event.file);
            this.snFileListChange.emit(this.imgList);
        }
        if(event.type === "error") {
            console.log('upload error =', event);
        }
    }

    remove = (file: UploadFile) => {
        // console.log('remove=file=', file);
        let index = this._snFileList.indexOf(file);
        this.imgList.splice(index, 1);
        this._snFileList.splice(index, 1);
        let imglist = [...this.imgList];
        this.snChangeCallback.emit(imglist);
        this.snRemoveCallback.emit(file);
        this.snFileListChange.emit(imglist);
        return;
    }
}
