import { Component, OnInit, ViewChild, ElementRef, EventEmitter, HostListener, Input, Output } from '@angular/core';

import { PAGEIMGLIST } from '../model/index';

@Component({
  selector: 'sn-page-preview',
  templateUrl: './page-preview.component.html',
  styleUrls: ['./page-preview.component.css']
})
export class PagePreviewComponent implements OnInit {

  focusElement: any;
  zoomRatio: number = 1.25;
  imgRatio: number;
  isSpinning: boolean = false;
  mouseDown: boolean = false;
  disX: number = 0;
  disY: number = 0;
  nextDeg: number = 0
  nowImgUrl: string;
  // nowImgIndex: number = 0;

  constructor() { }

  @Input() imgList: PAGEIMGLIST[] = [];
  @Input() snHeight?: string = '300px';
  @Input() snIndex: number = 0;
  @Output() snPageBtnClick = new EventEmitter<number>();

  @ViewChild("container") container: ElementRef;
  @ViewChild("imageInstance") imageInstance: ElementRef;

  ngOnInit() {
  }

  ngOnChanges() {
    console.log('ngOnChanges==');
    this.loadImg();
  }

  loadImg() {
    if (!this.imgList || this.imgList.length == 0) return;

    this.nowImgUrl = this.imgList[this.snIndex].url;
    this.isSpinning = true;
    setTimeout(() => {
      this.setImgSize();
    }, 1000);
  }

  @HostListener("mousedown", ["$event"])
  onMousedown(event: MouseEvent) {
    this.mouseDown = true;
    if (!this.imageInstance) return;
    if (event.target == this.imageInstance.nativeElement) {
      event.preventDefault();//禁止浏览器自带拖动效果
      this.focusElement = this.imageInstance.nativeElement;
      this.disX = event.clientX - this.imageInstance.nativeElement.offsetLeft;
      this.disY = event.clientY - this.imageInstance.nativeElement.offsetTop;
      this.imageInstance.nativeElement.setCapture && this.imageInstance.nativeElement.setCapture.setCapture();
    } else {
      this.mouseDown = false;
    }
  }
  @HostListener('mousemove', ['$event'])
  onMousemove(event: MouseEvent) {
    if (!this.imageInstance) return;
    if (this.mouseDown) {
      if (this.focusElement == this.imageInstance.nativeElement) {
        let iL = event.clientX - this.disX;
        let iT = event.clientY - this.disY;

        // iL <= 0 && (iL = 0);
        // iL > maxW && (iL = maxW);
        this.imageInstance.nativeElement.style.left = iL + "px";
        this.imageInstance.nativeElement.style.top = iT + "px";
      }
    }
  }
  @HostListener('window:mouseup')
  onMouseup() {
    if (!this.imageInstance) return;
    this.mouseDown = false;
    this.focusElement = null;
    document.onmousemove = null;
    document.onmouseup = null;
    this.imageInstance.nativeElement.releaseCapture && this.imageInstance.nativeElement.releaseCapture();
  };
  @HostListener("mousewheel", ["$event"])
  mousewheel(event: MouseEvent | any) {
    if (!this.imageInstance) return;
    if (event.target == this.imageInstance.nativeElement) {
      event.preventDefault();//禁止浏览器自带拖动效果

      if (event.wheelDelta > 0) {
        this.zoom();
      } else {
        this.zoomOut();
      }
      this.imageInstance.nativeElement.setCapture && this.imageInstance.nativeElement.setCapture.setCapture();
    }
  }

  setImgSize() {
    let containerWidth = this.container.nativeElement.clientWidth;
    let containerHeight = this.container.nativeElement.clientHeight;
    let imgNaturalWidth = this.imageInstance.nativeElement.naturalWidth;
    let imgNaturalHeight = this.imageInstance.nativeElement.naturalHeight;
    this.imgRatio = imgNaturalWidth / imgNaturalHeight;

    this.imageInstance.nativeElement.style.top = 0;
    this.imageInstance.nativeElement.className = "image";

    if (this.imgRatio < containerWidth / containerHeight) {
      let imgWidth = containerHeight / imgNaturalHeight * imgNaturalWidth;
      this.imageInstance.nativeElement.style.width = imgWidth + 'px';
      this.imageInstance.nativeElement.style.height = containerHeight + 'px';
      this.imageInstance.nativeElement.style.left = (containerWidth - imgWidth) / 2 + 'px';
    } else {
      let imgHeight = containerWidth / imgNaturalWidth * imgNaturalHeight;
      this.imageInstance.nativeElement.style.width = containerWidth + 'px';
      this.imageInstance.nativeElement.style.height = imgHeight + 'px';
      this.imageInstance.nativeElement.style.top = (containerHeight - imgHeight) / 2 + 'px';
    }

    this.isSpinning = false;
  }

  prev() {
    if (this.snIndex > 0) {
      this.isSpinning = true;
      this.snIndex--;
      this.nowImgUrl = this.imgList[this.snIndex].url;
      this.snPageBtnClick.emit(this.snIndex);

      setTimeout(() => {
        this.setImgSize();
      }, 1000);
    }
  }

  next() {
    if (this.snIndex < this.imgList.length - 1) {
      this.isSpinning = true;
      this.snIndex++;
      this.nowImgUrl = this.imgList[this.snIndex].url;
      this.snPageBtnClick.emit(this.snIndex);

      setTimeout(() => {
        this.setImgSize();
      }, 1000);
    }
  }

  zoom() {
    let newImgWidth = Math.ceil(this.imageInstance.nativeElement.clientWidth * this.zoomRatio);
    let newImgHeight = Math.ceil(this.imageInstance.nativeElement.clientHeight * this.zoomRatio);

    if(newImgWidth > newImgHeight && newImgWidth > 2000 || newImgWidth < newImgHeight && newImgHeight > 2000) return;

    this.updateImgSize('zoom', newImgWidth, newImgHeight);
  }

  zoomOut() {
    let newImgWidth = Math.ceil(this.imageInstance.nativeElement.clientWidth / this.zoomRatio);
    let newImgHeight = Math.ceil(this.imageInstance.nativeElement.clientHeight / this.zoomRatio);
    
    if(newImgWidth > newImgHeight && newImgWidth < 280 || newImgWidth < newImgHeight && newImgHeight < 280) return;

    this.updateImgSize('zoomout', newImgWidth, newImgHeight);
  }

  rotate() {
    let rotateClass = this.imageInstance.nativeElement.className.match(/(rotate)(\d*)/);
    if (rotateClass) {
      this.nextDeg = (rotateClass[2] * 1 + 90) % 360;
    } else {
      this.nextDeg = 90;
    }
    this.imageInstance.nativeElement.className = "image rotate" + this.nextDeg;

    this.imageInstance.nativeElement.style.left = (this.container.nativeElement.clientWidth - this.imageInstance.nativeElement.clientWidth) / 2 + 'px';
    this.imageInstance.nativeElement.style.top = (this.container.nativeElement.clientHeight - this.imageInstance.nativeElement.clientHeight) / 2 + 'px';
  }

  updateImgSize(type, newImgWidth, newImgHeight) {
    let containerWidth = this.container.nativeElement.clientWidth;
    let containerHeight = this.container.nativeElement.clientHeight;
    //   let isVertical = this.nextDeg == 90 || this.nextDeg == 270;
    let isVertical = true;

    this.imageInstance.nativeElement.style.width = newImgWidth + 'px';
    this.imageInstance.nativeElement.style.height = newImgHeight + 'px';

    if (containerWidth > isVertical ? newImgWidth : newImgHeight) {
      this.imageInstance.nativeElement.style.left = (containerWidth - newImgWidth) / 2 + 'px';
    } else {
      let iL = this.imageInstance.nativeElement.offsetTop + newImgWidth / 10 * (type === 'zoom' ? -1 : 1);
      let iR = containerWidth - newImgWidth;

      iL >= 0 && (iL = 0);
      iL <= iR && (iL = iR)

      this.imageInstance.nativeElement.style.left = iL + 'px';
    }
    if (containerHeight > isVertical ? newImgHeight : newImgWidth) {
      this.imageInstance.nativeElement.style.top = (containerHeight - newImgHeight) / 2 + 'px';
    } else {
      let iT = this.imageInstance.nativeElement.offsetLeft + newImgHeight / 10 * (type === 'zoom' ? -1 : 1);
      let iB = containerHeight - newImgHeight;

      iT >= 0 && (iT = 0);
      iT <= iB && (iT = iB)
      this.imageInstance.nativeElement.style.top = iT + "px";
    }
  }
}
