import { Component, OnInit, Input, Output, EventEmitter } from '@angular/core';
import { NzMessageService, UploadFile, UploadXHRArgs } from 'ng-zorro-antd';
import { SnFilerService } from '../sn-filer.service';
import { RESPONSE } from '../model';
import { HttpRequest, HttpClient, HttpEvent, HttpResponse, HttpEventType } from '@angular/common/http';

@Component({
  selector: 'sn-file-upload',
  templateUrl: './sn-file-upload.component.html',
  styleUrls: ['./sn-file-upload.component.css']
})
export class SnFileUploadComponent implements OnInit {

  action: string;
  uploadData: any = { "bucketAlias": "SmartEvent" };

  @Input() snListType: string = 'text';
  @Input() snUploadUrl: string; //上传地址域名
  @Input() snFileList: UploadFile[] = []; //文件列表
  @Input() snUploadData?: any = {
    url: "",
    params: {}
  }; //mapping参数
  @Input() snIsMapping: boolean = false; //是否上传完毕马上mapping
  @Input() snShowUploadList: boolean = true; //是否显示上传列表
  @Output() snFileListChange = new EventEmitter(); //返回文件列表
  @Output() snRemoveCallback = new EventEmitter(); //删除回调
  @Output() snUploadCallback = new EventEmitter(); //上传成功回调

  constructor(
    private nzMessageService: NzMessageService,
    private snFilerService: SnFilerService,
    private http: HttpClient
  ) { }

  ngOnInit() {
    this.action = this.snUploadUrl + "/foundation/aliyun/api/oss/upload";
  }

  changeCallback = (event: any) => {
    if (event.type === "success") {
      console.log('changeCallback==', event);
      let url = this.snUploadData.url; //完整的url路径
      let params = this.snUploadData.params;
      params.fileKeys = [event.file.response.content.fileKey];

      this.snFileList = event.fileList;

      if (this.snIsMapping) {
        this.nzMessageService.success('文件上传成功！数据绑定中...');
        this.snFilerService.mapping(url, params).subscribe((res: RESPONSE) => {
          if (res.success) {
            this.emitData(event.file);
            this.nzMessageService.success('绑定成功！');
          } else {
            this.snFileList = this.removeFileItem(event.file, this.snFileList);
            this.emitData(event.file);
            this.nzMessageService.error('绑定失败！');
          }
        });
      } else {
        this.emitData(event.file);
      }

    }
    if (event.type === "error") {
      console.log('upload error =', event);
      this.nzMessageService.error('上传失败，未知错误！');
    }
  }

  private emitData(file) {
    this.snFileListChange.emit(this.snFileList);
    this.snUploadCallback.emit(file);
  }

  private removeFileItem(file: UploadFile, fileList: UploadFile[]): UploadFile[] {
    return fileList.filter(item => (item.uid !== file.uid || item.fileKey !== file.fileKey));
  }

  remove = (file: UploadFile) => {
    let params = {
		  fileId: file.fileKey || (file.response && file.response.content.fileKey)
		}
    this.snFilerService.unmapFile(this.snUploadUrl, params).subscribe((res: RESPONSE) => {
      if (res.success) {
        this.snFileList = this.removeFileItem(file, this.snFileList);
        this.snRemoveCallback.emit(file);
        this.snFileListChange.emit(this.snFileList);
        this.nzMessageService.success('删除成功！');

        return true;
      }else {
        this.nzMessageService.error('删除失败！');
        return false;
      }
    });
  }

  handlePreview = (file: UploadFile) => {
    console.log('handlePreview==', file);
    let previewImage = file.url || file.response.content.url || file.response.content.urlThumbnail;
    // let previewImage = file.url || file.thumbUrl;

    window.open(previewImage, "_blank");
  }

  customReq = (item: UploadXHRArgs) => {
    console.log('customReq==', item);
    const isIMG = item.file.type === 'image/jpeg' || item.file.type === 'image/png';
    if(isIMG) {
      const _this = this;
      const ready = new FileReader();
      /*开始读取指定的Blob对象或File对象中的内容. 
      当读取操作完成时,readyState属性的值会成为DONE,如果设置了onloadend事件处理程序,则调用之.
      同时,result属性中将包含一个data: URL格式的字符串以表示所读取文件的内容.*/
      ready.readAsDataURL(item.file as any);
      ready.onload = function () {
        const path: any = this.result;
  
        const img = new Image();
        img.src = path;
        img.onload = function () {
          const newImgFile = _this.canvasDataURL(this);
  
          _this.uploadFile(item, newImgFile);
        }
      }        
    }else {
      this.uploadFile(item);
    }
  }

  /**
   * 压缩图片
   */
  canvasDataURL(that: any) {
    const imgWidth = that.width; //原图宽度
    const imgHeight = that.height; //原图高度
    //生成canvas
    const canvas = document.createElement('canvas');
    const ctx = canvas.getContext('2d');
    // 创建属性节点
    const anw = document.createAttribute("width");
    anw.nodeValue = imgWidth;
    const anh = document.createAttribute("height");
    anh.nodeValue = imgHeight;
    canvas.setAttributeNode(anw);
    canvas.setAttributeNode(anh);
    ctx.drawImage(that, 0, 0, imgWidth, imgHeight);

    const quality = 0.6;  // 默认图片压缩质量,值越小,所绘制出的图像越模糊
    const base64 = canvas.toDataURL('image/jpeg', quality); //压缩后base64 code

    return this.convertBase64UrlToBlob(base64);
  }

  /**
   * 将以base64的图片url数据转换为Blob
   * @param urlData
   *            用url方式表示的base64图片数据
   */
  convertBase64UrlToBlob(urlData: any) {
    var arr = urlData.split(','), mime = arr[0].match(/:(.*?);/)[1],
      bstr = atob(arr[1]), n = bstr.length, u8arr = new Uint8Array(n);
    while (n--) {
      u8arr[n] = bstr.charCodeAt(n);
    }
    return new Blob([u8arr], { type: mime });
  }

  /**
   * 文件上传
   */
  uploadFile = (item: UploadXHRArgs, file?: any) => {
    // 构建一个 FormData 对象，用于存储文件或其他参数
    const formData = new FormData();
    // tslint:disable-next-line:no-any
    formData.append('file', file || item.file);
    formData.append('bucketAlias', 'SmartEvent');
    const req = new HttpRequest('POST', item.action, formData, {
      reportProgress: true,
      // withCredentials: true
    });
    // 始终返回一个 `Subscription` 对象，nz-upload 会在适当时机自动取消订阅
    return this.http.request(req).subscribe((event: HttpEvent<{}>) => {
      if (event.type === HttpEventType.UploadProgress) {
        if (event.total > 0) {
          // tslint:disable-next-line:no-any
          (event as any).percent = event.loaded / event.total * 100;
        }
        // 处理上传进度条，必须指定 `percent` 属性来表示进度
        item.onProgress(event, file);
      } else if (event instanceof HttpResponse) {
        // 处理成功
        item.onSuccess(event.body, file, event);
      }
    }, (err) => {
      // 处理失败
      item.onError(err, file);
    });
  }

  beforeUpload = (file: File) => {
    if (this.snListType === 'text') {
      const isLt10M = file.size / 1024 / 1024 < 10;
      if (!isLt10M) {
        this.nzMessageService.error('上传文件需小于10MB!');
      }
      return isLt10M;
    } else if (this.snListType === 'picture-card') {
      const isIMG = file.type === 'image/jpeg' || file.type === 'image/png';
      if (!isIMG) {
        this.nzMessageService.error('只支持JPG或PNG格式!');
      }
      const isLt10M = file.size / 1024 / 1024 < 10;
      if (!isLt10M) {

      }
      return isIMG;
    };
  }

}
